package agent;

import queue.*;
import java.util.*;

public class MinConflictsSearch extends SearchMethod {
    
    protected int n;
    protected boolean[] conflicts;
    protected int queensInConflict;
    protected Hashtable visitedStates;
    
    public MinConflictsSearch(int n) {
        setN(n);
        visitedStates = new Hashtable();
    }
    
    protected Collection expand(Node node) {
        return null;
    }
    
    public Action[] search(Problem problem) {
        
        ArrayList actionList = new ArrayList();
        setProblem(problem);
        State state = problem.getInitialState();
        visitedStates.put(state.getKey(),state);
        
        int[] variables = (int[]) state.getVariables().clone();
        calculateQueenConflicts(variables);
        
        int conflictQueen = findConflict();
        int newPosition;
        
        while (queensInConflict > 0) {
            newPosition = calculateNewPosition(variables, conflictQueen);
            if (newPosition != variables[conflictQueen]) {
                variables = changeState(state, conflictQueen, newPosition);
                while (visitedStates.containsKey(state.getKey())) {
                    Action action = new Action(conflictQueen, newPosition);
                    actionList.add(action);
                    conflictQueen = chooseValue();
                    newPosition = chooseValue() + 1;
                    variables = changeState(state, conflictQueen, newPosition);
                }
                visitedStates.put(state.getKey(),state);
                Action action = new Action(conflictQueen, newPosition);
                actionList.add(action);
            }
            conflictQueen = findConflict(variables, conflictQueen, newPosition);
            if (conflictQueen < 0 && !problem.goalTest(state)) {
                conflictQueen = findConflict();
            }
        }
        
        Action[] actions = new Action[actionList.size()];
        for (int i=0; i<actions.length; i++) {
            actions[i] = (Action) actionList.get(i);
        }
        return actions;
    }
    
    //reposiciona a rainha para a nova posicao, diminuindo o conflito.
    private int[] changeState(State state, int conflictQueen, int newPosition) {
        int[] variables = state.getVariables();
        variables[conflictQueen] = newPosition;
        calculateQueenConflicts(variables);
        return variables;
    }
    
    //escolhe aleatoriamente um valor no array
    private int choose(int[] values) {
        if (values == null) {
            return -1;
        }
        int num = (int)(Math.random() * values.length);
        return values[num];
    }
    
    //escolhe aleatoriamente um valor
    private int chooseValue() {
        return (int)(Math.random() * n);
    }
    
    //determina as rainhas em conflito
    private boolean[] calculateQueenConflicts(int[] variables) {
        conflicts = new boolean[variables.length];
        queensInConflict = 0;
        int nc = 0;
        int increment;
        for (int i=0; i<variables.length; i++) {
            increment = 1;
            for (int j=i+1; j<variables.length; j++) {
                if ((variables[i] == variables[j] - increment) || (variables[i] == variables[j]) || (variables[i] == variables[j] + increment)) {
                    
                    //Conflito encontrado: rainha i ataca j
                    
                    //se a rainha i não está em conflito com nenhuma outra, incrementa queensInConflict
                    if (!conflicts[i]) {
                        queensInConflict++;
                        conflicts[i] = true;
                    }
                    
                    //se a rainha j não está em conflito com nenhuma outra, incrementa queensInConflict
                    if (!conflicts[j]) {
                        queensInConflict++;
                        conflicts[j] = true;
                    }
                }
                increment++;
            }
        }
        return conflicts;
    }
    
    public int calculateNewPosition(int[] variables, int conflictQueen) {
        int originalPosition = variables[conflictQueen];
        
        int[] minConflicts = new int[variables.length];
        int aux = 0;
        int leastNc = variables.length;
        for (int i=0; i<variables.length; i++) {
            variables[conflictQueen] = i+1;
            //calculateQueenConflicts(variables);
            if (queensInConflict < leastNc) {
                minConflicts = new int[variables.length];
                aux = 0;
                minConflicts[aux++] = i + 1;
                leastNc = queensInConflict;
            } else if (queensInConflict == leastNc) {
                minConflicts[aux++] = i + 1;
                leastNc = queensInConflict;
            }
        }
        variables[conflictQueen] = originalPosition;
        
        minConflicts = trim(minConflicts);
        int newPosition = -1;
        if (minConflicts.length > 1) {
            newPosition = choose(minConflicts);
        } else if (minConflicts.length == 1) {
            newPosition = minConflicts[0];
        }
        return newPosition;
    }
    
    //dado um array {a1,a2,...,an,0,0,0...0} com a1..an < 0, retorna o array {a1,a2,...,an);
    public int[] trim(int[] array) {
        int size = 0;
        while (size < array.length && array[size] > 0) {
            size++;
        }
        int[] newArray = new int[size];
        if (size==array.length) {
            return array;
        } else {
            for (int i=0; i<size; i++) {
                newArray[i] = array[i];
            }
        }
        return newArray;
    }
    
    private int findConflict(int[] variables, int conflictQueen, int newPosition) {
        int increment;
        for (int i=0; i<variables.length; i++) {
            increment = 1;
            if ((variables[i] == variables[conflictQueen] - increment) || (variables[i] == variables[conflictQueen]) || (variables[i] == variables[conflictQueen] + increment)) {
                if (i != conflictQueen) {
                    //Conflito encontrado: rainha i ataca conflictQueen
                    return i;
                }
            }
            increment++;
        }
        return -1;
    }
    
    //retorna um conflito (escolha aleatoria).
    private int findConflict() {
        int conflictQueen = -1;
        if (queensInConflict > 0) {
            int[] conflictQueens = new int[queensInConflict];
            int aux = 0;
            for (int i=0; i<conflicts.length; i++) {
                if (conflicts[i]) {
                    conflictQueens[aux] = i;
                    aux++;
                }
            }
            int rand = (int) (Math.random() * queensInConflict);
            conflictQueen = conflictQueens[rand];
        }
        return conflictQueen;
    }
    
    /** Getter for property n.
     * @return Value of property n.
     */
    public int getN() {
        return n;
    }
    
    /** Setter for property n.
     * @param n New value of property n.
     */
    public void setN(int n) {
        this.n = n;
    }
}
