package agent;

import java.util.*;

public class NQueenSolverAgent {
    
    public static final int N_QUEEN_INCREMENTAL = 0;
    public static final int N_QUEEN_QUICK_INCREMENTAL = 1;
    public static final int N_QUEEN_COMPLETE_STATE = 2;
    
    private int formulationKind;
    private int n;
    private Problem problem;
    private SearchMethod searchMethod;
    
    public NQueenSolverAgent() {
        this.n = 8;
        this.formulationKind = N_QUEEN_INCREMENTAL;
        setProblem(new NQueenProblem(this.n, this.formulationKind));
        setupSearchMethod();
    }
    
    public NQueenSolverAgent(int n, int formulationKind) {
        setN(n);
        setFormulationKind(formulationKind);
    }
    
    private void setupSearchMethod() {
        switch (formulationKind) {
            case N_QUEEN_INCREMENTAL :
                searchMethod = (new ForwardCheckingSearch(n));
                break;
            case N_QUEEN_QUICK_INCREMENTAL :
                searchMethod = (new ForwardCheckingSearch(n, ForwardCheckingSearch.QUICK_SEARCH));
                break;
            case N_QUEEN_COMPLETE_STATE :
                searchMethod = (new MinConflictsSearch(n));
                break;
        }
    }
    
    public Action[] solve() {
        return searchMethod.search(problem);
    }
    
    public Collection getAllSolutions() {
        ArrayList allSolutions = new ArrayList();
        Action[] actionSequence = searchMethod.search(problem);
        while (actionSequence != null) {
            allSolutions.add(actionSequence);
            actionSequence = searchMethod.search(problem);
        }
        return allSolutions;
    }
    
    public State getInitialState() {
        return problem.getInitialState();
    }
    
    public void setInitialState(State initialState) {
        problem.setInitialState(initialState);
    }
    
    /** Getter for property formulationKind.
     * @return Value of property formulationKind.
     */
    public int getFormulationKind() {
        return formulationKind;
    }
    
    /** Setter for property formulationKind.
     * @param formulationKind New value of property formulationKind.
     */
    public void setFormulationKind(int formulationKind) {
        this.formulationKind = formulationKind;
        setProblem(new NQueenProblem(this.n, this.formulationKind));
    }
    
    /** Getter for property n.
     * @return Value of property n.
     */
    public int getN() {
        return n;
    }
    
    /** Setter for property n.
     * @param n New value of property n.
     */
    public void setN(int n) {
        this.n = n;
        setProblem(new NQueenProblem(this.n, this.formulationKind));
        setupSearchMethod();
    }
    
    /** Getter for property problem.
     * @return Value of property problem.
     */
    public agent.Problem getProblem() {
        return problem;
    }
    
    /** Setter for property problem.
     * @param problem New value of property problem.
     */
    public void setProblem(agent.Problem problem) {
        this.problem = problem;
        setupSearchMethod();
    }
}
